using System;
using System.Collections;


namespace HTMLParser
{
	
	/// <summary>
	/// Summary description for Node.
	/// </summary>
	public class Node
	{
		
		public const int TAGTYPE = 1;
		public const int TEXTTYPE = 2;
		int nodeType; // TAGTYPE or TEXTTYPE
		string type; // Tag type, empty for text node
		string text; // Empty for tag nodes
		bool isEndTag;

		Hashtable param = new Hashtable();
		ArrayList subNodes = new ArrayList();

		public bool IsEndTag
		{
			get
			{
				return isEndTag;
			}
		}
		public int Type
		{
			get
			{
				return nodeType;
			}
		}

		public string Tag
		{
			get
			{
				return type;
			}
		}

		public string Text
		{
			get
			{
				return text;
			}
		}

		public ArrayList SubNodes
		{
			get
			{
				return subNodes;
			}
			set
			{
				subNodes = value;
			}
		}

		public string ParamString
		{
			get
			{
				string ret = "";
				foreach(string key in param.Keys)
				{
					string val = (string)param[key];
					if(val.IndexOf('\"') == -1)
						ret += " " + key + " = \"" + val + "\"";
					else 
						ret += " " + key + " = '" + val + "'";
				}
				return ret;
			}
		}

		private Node()
		{
		}

		private void parseParams(string parms)
		{
			int p = 0;
			while(p < parms.Length)
			{
				while(char.IsWhiteSpace(parms, p)) p++;
				int sp = p;
				while(parms[p] != ' ' && parms[p] != '=') p++;
				
				string pname = parms.Substring(sp, p-sp);
				
				while(parms[p] != '=') p++;
				
				p++;

				while(char.IsWhiteSpace(parms, p)) p++;

				sp = p;
				char endChar = ' ';
				
				if((parms[p] == '\"') || (parms[p] == '\''))
				{
					sp ++;
                    endChar = parms[p];
					p++;
				}

				while((p != parms.Length) && (parms[p] != endChar)) p++;

				string pvalue = parms.Substring(sp, p - sp);

				param[pname] = pvalue;
				p++;

			}
		}

		public static Node MakeTag(string type, string parms, bool endTag)
		{
			Node n = new Node();
			n.nodeType = TAGTYPE;
			n.type = type;
			n.parseParams(parms);
			n.isEndTag = endTag;
			return n;
		}

		public static Node MakeText(string text)
		{
			Node n = new Node();
			text.Replace("\r\n", " ");
			for(int i = 0; i < text.Length - 1; i++)
			{
				while(i < text.Length - 1 && char.IsWhiteSpace(text, i) && char.IsWhiteSpace(text, i + 1))
				{
                    text = text.Remove(i + 1, 1);
				}
			}

			n.text = text;
			n.nodeType = TEXTTYPE;
			return n;
		}

		public void MakeTree()
		{
			// This searches the subnodes list for nested nodes
			for(int i = 0; i < subNodes.Count; i++)
			{
				Node cur = ((Node)subNodes[i]);
				if((cur.Type == TAGTYPE) && (cur.isEndTag == false))
				{
					int level = 1;
					for(int j = i + 1; j < subNodes.Count; j++)
					{
						Node cur2 = (Node)subNodes[j];
						if( (cur2.Tag == cur.Tag) )
						{
							if( (cur2.isEndTag == true) )
							{
								level --;
							} 
							else 
							{
								level ++;
							}

							if( level == 0)
							{
								for(int k = i + 1; k < j; k++)
								{
									cur.subNodes.Add(subNodes[i+1]);
									subNodes.RemoveAt(i+1);
								}
								subNodes.RemoveAt(i+1);
								cur.MakeTree();
								break;
							}
						}
					}
				}

			}
		}
	}
}
